import math
import re


def cosine_similarity(vec1, vec2):
    dot_product = sum([x * y for x, y in zip(vec1, vec2)])
    norm1 = math.sqrt(sum([x ** 2 for x in vec1]))
    norm2 = math.sqrt(sum([x ** 2 for x in vec2]))
    norm_product = norm1 * norm2
    if norm_product == 0:
        return 0
    else:
        return dot_product / norm_product


def is_image_url(url: str) -> bool:
    return any(re.search(pattern, url, re.IGNORECASE) for pattern in ['.jpg', '.jpeg', '.png', '.gif'])



import httpx


def _send_to_openai(endpoint_url: str,):
    async def send_to_openai(api_key: str, timeout: float, payload: dict) -> httpx.Response:
        """
        Send a request to openai.
        :param api_key: your api key
        :param timeout: timeout in seconds
        :param payload: the request body, as detailed here: https://beta.openai.com/docs/api-reference
        """
        async with httpx.AsyncClient() as client:
            return await client.post(
                url=endpoint_url,
                json=payload,
                headers={"content_type": "application/json", "Authorization": f"Bearer {api_key}"},
                timeout=timeout,
            )

    return send_to_openai


complete = _send_to_openai("https://api.openai.com/v1/completions")
generate_img = _send_to_openai("https://api.openai.com/v1/images/generations")
embeddings = _send_to_openai("https://api.openai.com/v1/embeddings")



import pytest
import openai_async
import os
from openai_async import test_utils

_OPEN_AI_API_KEY = os.getenv("OPENAI_API_KEY")


@pytest.mark.asyncio
async def test_complete():
    response = await openai_async.complete(
        _OPEN_AI_API_KEY,
        timeout=2,
        payload={
            "model": "text-davinci-003",
            "prompt": "How much is 1 + 2?",
            "temperature": 0.0,
        },
    )
    assert response.json()["choices"][0]["text"].strip() == "1 + 2 = 3"


@pytest.mark.asyncio
async def test_generate_img():
    response = await openai_async.generate_img(
        _OPEN_AI_API_KEY,
        timeout=8,
        payload={
            "prompt": "a white siamese cat",
            "n": 1,
            "size": "256x256"
        },
    )
    assert test_utils.is_image_url(response.json()["data"][0]["url"])


@pytest.mark.asyncio
async def test_embeddings():
    response1 = await openai_async.embeddings(
        _OPEN_AI_API_KEY,
        timeout=2,
        payload={"model": "text-embedding-ada-002", "input": "tooth doctor"},
    )
    response2 = await openai_async.embeddings(
        _OPEN_AI_API_KEY,
        timeout=2,
        payload={"model": "text-embedding-ada-002", "input": "dentist"},
    )
    assert (
        test_utils.cosine_similarity(
            response1.json()["data"][0]["embedding"],
            response2.json()["data"][0]["embedding"],
        )
        > 0.9
    )



import setuptools

with open("README.md", "r") as fh:
    _LONG_DESCRIPTION = fh.read()


setuptools.setup(
    name="openai-async",
    license='MIT',
    author="itayzit",
    version="0.0.1",
    author_email="itayzit@gmail.com",
    long_description=_LONG_DESCRIPTION,
    long_description_content_type="text/markdown",
    url="https://github.com/itayzit/openai-async",
    packages=setuptools.find_namespace_packages(),
    install_requires=["httpx", "pytest"],
)